@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1env.c 
   ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        1999-08-12
+  ----- Date:        2000-01-12
   ----- Description: This file is part of the t1-library. It implements
                      the reading of a configuration file and path-searching
 		     of type1-, afm- and encoding files.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1998. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2000. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -33,7 +33,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#if defined(_MSC_VER)
+# include <io.h>
+# include <sys/types.h>
+# include <sys/stat.h>
+#else
+# include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <ctype.h>
@@ -47,6 +53,7 @@
 #include "../type1/fontfcn.h"
 #include "../type1/fontmisc.h"
 
+#include "sysconf.h"
 #include "t1types.h"
 #include "t1extern.h"
 #include "t1env.h"
@@ -63,12 +70,13 @@ int ScanConfigFile( char **pfabenv_ptr,
 		    char **fontdatabase_ptr)
 {
   
-  char *environ;
+  char *env_str;
   char *linebuf;
   char *usershome;
   char *cnffilepath;
   char *globalcnffilepath;
   char *tmp_ptr;
+  static int linecnt;
   
   
   
@@ -77,10 +85,10 @@ int ScanConfigFile( char **pfabenv_ptr,
   
   
   /* First, get the string stored in the environment variable: */
-  environ=getenv(ENV_CONF_STRING);
+  env_str=getenv(ENV_CONF_STRING);
+  linecnt=1;
 
-
-  if (environ==NULL) {
+  if (env_str==NULL) {
     /* environment variable not set, try to open default file
        in user's home directory and afterwards global config file */
     if ((usershome=getenv("HOME"))!=NULL) {
@@ -143,7 +151,7 @@ int ScanConfigFile( char **pfabenv_ptr,
   }
   else{
     /* open specified file for reading the configuration */
-    if ((cfg_fp=fopen(environ,"r"))==NULL){
+    if ((cfg_fp=fopen(env_str,"r"))==NULL){
       T1_PrintLog( "ScanConfigFile()",
 		   "Configfile as specified by Environment has not been found",
 		   T1LOG_WARNING);
@@ -152,7 +160,7 @@ int ScanConfigFile( char **pfabenv_ptr,
     }
     else{
       sprintf( err_warn_msg_buf, "Using %s as Configfile (environment)",
-	       environ);
+	       env_str);
       T1_PrintLog( "ScanConfigFile()", err_warn_msg_buf, T1LOG_STATISTIC);
     }
   }
@@ -165,7 +173,7 @@ int ScanConfigFile( char **pfabenv_ptr,
   /* Reset fileposition to start */
   fseek( cfg_fp, 0, SEEK_SET);
   
-  if ((linebuf=(char *)calloc( filesize,
+  if ((linebuf=(char *)calloc( filesize+1,
 			       sizeof(char)))==NULL){
     T1_errno=T1ERR_ALLOC_MEM;
     return(-1);
@@ -178,22 +186,25 @@ int ScanConfigFile( char **pfabenv_ptr,
   
   while(i<filesize){
     j=i;     /* Save index of beginning of line */
-    while ((linebuf[i]!='=')&&(linebuf[i]!='\n'))
+    while ((linebuf[i]!='=')&&(linebuf[i]!='\n')&&(i<filesize))
       i++;
-    linebuf[i]=0;  /* replace '=' by ASCII-0 */
-    if (strcmp( "ENCODING", &linebuf[j])==0){
+    if (i==filesize) {
+      free( linebuf);
+      return(i);
+    }
+    if (strncmp( "ENCODING", &linebuf[j], 8)==0) {
       /* Check for an explicitly assigned value */
       if (*encenv_ptr==T1_enc){
 	k=i+1;       /* points to assigned path string */
-	while (linebuf[i] != '\n')
+	while ((!isspace((int)linebuf[i]))&&(i<filesize))
 	  i++;
-	linebuf[i]=0;  /* replace linefeed by ASCII-0 */
 	/* Copy string */
-	if ((tmp_ptr=(char *)malloc( strlen(&linebuf[k]) + 1))==NULL){
+	if ((tmp_ptr=(char *)malloc( i-k+1))==NULL){
 	  T1_errno=T1ERR_ALLOC_MEM;
 	  return(-1);
 	}
-	strcpy( tmp_ptr, &linebuf[k]);
+	strncpy( tmp_ptr, &linebuf[k], i-k);
+	tmp_ptr[i-k]='\0';
 	*encenv_ptr=tmp_ptr;
       }
       else
@@ -201,19 +212,19 @@ int ScanConfigFile( char **pfabenv_ptr,
 		     "Preserving explicitly assigned ENCODING search path",
 		     T1LOG_DEBUG);
     }  
-    if (strcmp( "TYPE1", &linebuf[j])==0){
+    else if (strncmp( "TYPE1", &linebuf[j], 5)==0) {
       /* Check for an explicitly assigned value */
       if (*pfabenv_ptr==T1_pfab){
 	k=i+1;       /* points to assigned path string */
-	while (linebuf[i] != '\n')
+	while ((!isspace((int)linebuf[i]))&&(i<filesize))
 	  i++;
-	linebuf[i]=0;  /* replace linefeed by ASCII-0 */
 	/* Copy string */
-	if ((tmp_ptr=(char *)malloc( strlen(&linebuf[k]) + 1))==NULL){
+	if ((tmp_ptr=(char *)malloc( i-k+1))==NULL){
 	  T1_errno=T1ERR_ALLOC_MEM;
 	  return(-1);
 	}
-	strcpy( tmp_ptr, &linebuf[k]);
+	strncpy( tmp_ptr, &linebuf[k], i-k);
+	tmp_ptr[i-k]='\0';
 	*pfabenv_ptr=tmp_ptr;
       }
       else
@@ -221,19 +232,19 @@ int ScanConfigFile( char **pfabenv_ptr,
 		     "Preserving explicitly assigned PFAB search path",
 		     T1LOG_DEBUG);
     }  
-    if (strcmp( "AFM", &linebuf[j])==0){
+    else if (strncmp( "AFM", &linebuf[j], 3)==0) {
       /* Check for an explicitly assigned value */
       if (T1_AFM_ptr==T1_afm){
 	k=i+1;       /* points to assigned path string */
-	while (linebuf[i] != '\n')
+	while ((!isspace((int)linebuf[i]))&&(i<filesize))
 	  i++;
-	linebuf[i]=0;  /* replace linefeed by ASCII-0 */
 	/* Copy string */
-	if ((tmp_ptr=(char *)malloc( strlen(&linebuf[k]) + 1))==NULL){
+	if ((tmp_ptr=(char *)malloc( i-k+1))==NULL){
 	  T1_errno=T1ERR_ALLOC_MEM;
 	  return(-1);
 	}
-	strcpy( tmp_ptr, &linebuf[k]);
+	strncpy( tmp_ptr, &linebuf[k], i-k);
+	tmp_ptr[i-k]='\0';
 	*afmenv_ptr=tmp_ptr;
       }
       else
@@ -241,28 +252,19 @@ int ScanConfigFile( char **pfabenv_ptr,
 		     "Preserving explicitly assigned AFM search path",
 		     T1LOG_DEBUG);
     }  
-    if (strcmp( "FONTDATABASE", &linebuf[j])==0){
+    else if (strncmp( "FONTDATABASE", &linebuf[j], 12)==0) {
       /* Check for an explicitly assigned value */
       if (*fontdatabase_ptr==T1_fontdatabase){
 	k=i+1;       /* points to assigned path string */
-	while (!isspace((int)linebuf[i]))
+	while ((!isspace((int)linebuf[i]))&&(i<filesize))
 	  i++;
-	if (linebuf[i]=='\n'){
-	  linebuf[i]=0;  /* replace linefeed by ASCII-0 */
-	}
-	else{
-	  linebuf[i]=0;  /* replace linefeed by ASCII-0 */
-	  /* Step to end of line */
-	  while (linebuf[i] != '\n')
-	    i++;
-	  linebuf[i]=0;
-	}
 	/* Copy string */
-	if ((tmp_ptr=(char *)malloc( strlen(&linebuf[k]) + 1))==NULL){
+	if ((tmp_ptr=(char *)malloc( i-k+1))==NULL){
 	  T1_errno=T1ERR_ALLOC_MEM;
 	  return(-1);
 	}
-	strcpy( tmp_ptr, &linebuf[k]);
+	strncpy( tmp_ptr, &linebuf[k], i-k);
+	tmp_ptr[i-k]='\0';
 	*fontdatabase_ptr=tmp_ptr;
       }
       else
@@ -270,7 +272,16 @@ int ScanConfigFile( char **pfabenv_ptr,
 		     "Preserving explicitly assigned FontDataBase",
 		     T1LOG_DEBUG);
     }
+    else {
+      sprintf( err_warn_msg_buf, "Ignoring line %d", linecnt);
+      T1_PrintLog( "ScanConfigFile()", err_warn_msg_buf,
+		   T1LOG_DEBUG);
+    }
+    
+    while ((linebuf[i]!='\n')&&(i<filesize))
+      i++;
     i++;
+    linecnt++;
   }
   /*file should now be read in */
   free( linebuf);
@@ -311,6 +322,9 @@ char *Env_GetCompletePath( char *FileName,
 #ifdef __EMX__
        ||
        ((isalpha(FileName[0])) && (FileName[1]==':'))
+#endif
+#ifdef VMS
+       || (strchr(FileName,':') != NULL)
 #endif
        )
     {
@@ -373,6 +387,12 @@ char *Env_GetCompletePath( char *FileName,
     /* Copy current path element: */
     res_ptr=strcpy( FullPathName, &env_ptr[j]);
     /* Add the directory separator: */
+#ifdef VMS
+    { char *p= strrchr(FullPathName, DIRECTORY_SEP_CHAR);
+      if (p && *(p+1) ==  '\0')
+       *p = '\0';
+    } 
+#endif 
     res_ptr=strcat( FullPathName, DIRECTORY_SEP);
     /* And finally the filename: */
     res_ptr=strcat( FullPathName, StrippedName);
@@ -537,7 +557,7 @@ int T1_AddToFileSearchPath( int pathtype, int mode, char *pathname)
       strcat(tmp_ptr, PATH_SEP);
       strcat(tmp_ptr, pathname);
     }
-    if ((void *)T1_PFAB_ptr!=(void *)&T1_pfab)
+    if ((void *)T1_PFAB_ptr!=(void *)T1_pfab)
       free( T1_PFAB_ptr);
     T1_PFAB_ptr=tmp_ptr;
   }
@@ -558,7 +578,7 @@ int T1_AddToFileSearchPath( int pathtype, int mode, char *pathname)
       strcat(tmp_ptr, PATH_SEP);
       strcat(tmp_ptr, pathname);
     }
-    if ((void *)T1_AFM_ptr!=(void *)&T1_afm)
+    if ((void *)T1_AFM_ptr!=(void *)T1_afm)
       free( T1_AFM_ptr);
     T1_AFM_ptr=tmp_ptr;
   }
@@ -579,7 +599,7 @@ int T1_AddToFileSearchPath( int pathtype, int mode, char *pathname)
       strcat(tmp_ptr, PATH_SEP);
       strcat(tmp_ptr, pathname);
     }
-    if ((void *)T1_ENC_ptr!=(void *)&T1_enc)
+    if ((void *)T1_ENC_ptr!=(void *)T1_enc)
       free( T1_ENC_ptr);
     T1_ENC_ptr=tmp_ptr;
   }
