@@ -35,7 +35,9 @@
 #include "token.h"
 #include "fontfcn.h"
 #include "blues.h"
- 
+
+
+/* #define DEBUG_SCANFONT */
  
  
 static int rc;
@@ -50,7 +52,8 @@ static psobj *inputP;
 extern int T1Getc(F_FILE * );
 extern int T1Ungetc( int,F_FILE *);
 extern int SearchDictName( psdict *dictP, psobj *keyP);
-  
+extern void T1io_setblocklen( long len);
+
  
 /* functions from util.c */
 extern void objFormatInteger(psobj *objP, int value);
@@ -76,7 +79,7 @@ typedef struct				/* Builtin Standard Encoding */
 } EncodingTable;
 
 
-static char not_def[]=".notdef";
+char not_def[]=".notdef";
 
 
 static EncodingTable StdEnc[] = {
@@ -265,7 +268,11 @@ static psobj *MakeEncodingArrayP(encodingTable)
 boolean Init_BuiltInEncoding()
 {
   StdEncArrayP = MakeEncodingArrayP(StdEnc);
-  return ((boolean)StdEncArrayP);
+  if (StdEncArrayP) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
 }
  
 /********************************************************************/
@@ -364,7 +371,7 @@ static int getEncoding(arrayP)
       arrayP->len = 256;
 
       for (i=0; i<256; i++)
-	  objFormatName(objP + i, 7, ".notdef");
+	  objFormatName(objP + i, 7, not_def);
 
       while (TRUE)
       {
@@ -456,20 +463,24 @@ static int getArray(arrayP)
   /* NOTREACHED*/
 }
 /***================================================================***/
-static int getName(nameP)
-    char *nameP;
-{
-  do {
-    scan_token(inputP);
-    if (tokenType <= TOKEN_NONE) {
-      if (tokenTooLong) return(SCAN_OUT_OF_MEMORY);
-      return(SCAN_ERROR);
-    }
+/* is not needed */
+/*
+  static int getName(nameP)
+  char *nameP;
+  {
+  do { 
+  scan_token(inputP);
+  if (tokenType <= TOKEN_NONE) {
+  if (tokenTooLong) return(SCAN_OUT_OF_MEMORY);
+  return(SCAN_ERROR);
+  }
   } while ((tokenType != TOKEN_NAME) ||
-    (0 != strncmp(tokenStartP,nameP,strlen(nameP))) );
-  /* found */
+  (0 != strncmp(tokenStartP,nameP,strlen(nameP))) );
+  
   return(SCAN_OK);
-}
+  }
+*/
+
 /***================================================================***/
 static int getNbytes(N)
     int N;
@@ -517,7 +528,7 @@ static int getLiteralName(nameObjP)
  
 /***================================================================***/
 /*
- *   BuildSubrs routine
+ *   BuildSubrs routine 
  */
 /***================================================================***/
  
@@ -542,8 +553,19 @@ static int BuildSubrs(FontP)
      /* process all the Subrs, but do not update anything */
      /* can not just skip them because of the binary data */
      for (i=0;i<N;i++) {
-       /* look for dup */
-       rc = getName("dup");
+       /* look for "dup" or "ND" or "|-" or (noaccess) "def". If one of
+	  the latter three token appears, we are done even if still i < N-1.
+	  in this case, there arent as much subroutines as fields in the
+	  array which is allowed (Fix thanks to Derek B. Noonburg) */
+       do {
+	 rc = getNextValue(TOKEN_NAME);
+	 if ( rc != SCAN_OK ) return(rc);
+	 if (!strncmp(tokenStartP, "ND", 2) ||
+	     !strncmp(tokenStartP, "|-", 2) ||
+	     !strncmp(tokenStartP, "def", 3)) {
+	   return SCAN_OK;
+	 }
+       } while (strncmp(tokenStartP, "dup", 3));
        if (rc) return(rc);
        /* get 2 integers */
        I = getInt();
@@ -562,17 +584,30 @@ static int BuildSubrs(FontP)
  
    arrayP = (psobj *)vm_alloc(N*sizeof(psobj));
    if (!(arrayP) ) return(SCAN_OUT_OF_MEMORY);
+   /* explicitly resetting the array is not necessary since the pointer
+      as well as the subroutines both are located in VM and we can
+      assume that only define subroutines are referenced. */
    FontP->Subrs.len = N;
    FontP->Subrs.data.arrayP =  arrayP;
    /* get N values for Subrs */
    for (i=0;i<N;i++) {
-     /* look for dup */
-     rc = getName("dup");
-     if (rc) return(rc);
+     /* look for "dup" or "ND" or "|-" or (noaccess) "def". If one of
+	the latter three token appears, we are done even if still i < N-1.
+	in this case, there arent as much subroutines as fields in the
+	array which is allowed (Fix thanks to Derek B. Noonburg) */
+     do {
+       rc = getNextValue(TOKEN_NAME);
+       if ( rc != SCAN_OK ) return(rc);
+       if (!strncmp(tokenStartP, "ND", 2) ||
+	   !strncmp(tokenStartP, "|-", 2) ||
+	   !strncmp(tokenStartP, "def", 3)) {
+	 return SCAN_OK;
+       }
+     } while (strncmp(tokenStartP, "dup", 3));
      /* get 2 integers */
-     I = getInt();
-     if (rc) return(rc);
-     J = getInt();
+     I = getInt();       /* index into array of Subroutines */
+     if (rc) return(rc); 
+     J = getInt();       /* number of binary bytes that follow */
      if (rc) return(rc);
      if ( (I < 0) || (J < 0 ) ) return (SCAN_ERROR);
      arrayP[I].len = J;
@@ -589,6 +624,8 @@ static int BuildSubrs(FontP)
    return(SCAN_OK);
  
 }
+
+
 /***================================================================***/
 /***================================================================***/
 /*
@@ -631,6 +668,8 @@ static int BuildCharStrings(FontP)
    for (i=1;i<=N;i++) {
      /* look for next literal name  */
      rc = getLiteralName(&(dictP[i].key));
+     /* If rc=SCAN_END, the end of file has been reached. This means
+	there were less than N charstrings. This is perfectly valid. */
      if (rc) return(rc);
      /* get 1 integer */
      J = getInt();
@@ -645,7 +684,10 @@ static int BuildCharStrings(FontP)
        dictP[i].value.data.valueP = tokenStartP;
        if ( !(vm_alloc(J)) ) return(SCAN_OUT_OF_MEMORY);
      }
-     else return(rc);
+     else {
+       return(rc);
+     }
+     
    }
    return(SCAN_OK);
  
@@ -1181,11 +1223,7 @@ int scan_font(FontP)
   FILE   *fileP;
   char   *nameP;
   int    namelen;
-  int    V;
-  int    i;
-  boolean starthex80;
  
-    starthex80 = FALSE;
     filetype[0] = 'r';
     filetype[1] = 'b';
     filetype[2] = '\0';
@@ -1209,14 +1247,6 @@ int scan_font(FontP)
  
     inputP = &inputFile;
     if ((fileP = fopen(filename,filetype))!=NULL) {
-      /* get the first byte of file */
-      V = getc(fileP);
-      /* if file starts with x'80' then skip next 5 bytes */
-      if ( V == 0X80 ) {
-        for (i=0;i<5;i++) V = getc(fileP);
-        starthex80 = TRUE;
-      }
-      else ungetc(V,fileP);
       objFormatFile(inputP,fileP);
     }
     else {
@@ -1227,6 +1257,9 @@ int scan_font(FontP)
   InPrivateDict = FALSE;
   TwoSubrs      = FALSE;
   rc = BuildFontInfo(FontP);
+#ifdef DEBUG_SCANFONT
+  printf("BuildFontInfo(): retval=%d\n", rc);
+#endif
   if (rc != 0) return(rc);
  
   /* Assume everything will be OK */
@@ -1236,7 +1269,7 @@ int scan_font(FontP)
   do {
     /* Scan the next token */
     scan_token(inputP);
- 
+
     /* ==> tokenLength, tokenTooLong, tokenType, and tokenValue are */
     /* now set */
  
@@ -1251,17 +1284,33 @@ int scan_font(FontP)
       case TOKEN_LITERAL_NAME:
             /* Look up the name */
             tokenStartP[tokenLength] = '\0';
+	    /* At this point we check for the font not being a
+	       Multiple Master Font. If it is, we return an error.
+	       (RMz, 01/29/1999) */
+	    if (strncmp(tokenStartP, "BlendAxisTypes", 14)==0){
+	      rc=SCAN_MMFONT;
+	      break;
+	    }
             if (InPrivateDict ) {
               if (0== strncmp(tokenStartP,"Subrs",5) ) {
                 rc = BuildSubrs(FontP);
+#ifdef DEBUG_SCANFONT
+		printf("BuildSubrs(): retval=%d\n", rc);
+#endif
                 break;
               }
               if (0== strncmp(tokenStartP,"CharStrings",11) ) {
                 rc = BuildCharStrings(FontP);
+#ifdef DEBUG_SCANFONT
+		printf("BuildCharstrings(): retval=%d\n", rc);
+#endif
                 if ( (rc == SCAN_OK) ||(rc == SCAN_END) ) {
                   fclose(inputP->data.fileP);
                   /* Build the Blues Structure */
 		  rc = GetType1Blues(FontP);
+#ifdef DEBUG_SCANFONT
+		  printf("GetType1Blues(): retval=%d\n", rc);
+#endif
                   /* whatever the return code, return it */
                   /* all the work is done. This is the normal exit.*/
                   return(rc);
@@ -1295,22 +1344,15 @@ int scan_font(FontP)
         break;
       case TOKEN_NAME:
             if (0 == strncmp(tokenStartP,"eexec",5) ) {
-               /* if file started with x'80', check next 5 bytes */
-               if (starthex80) {
-                 V = getc(fileP);
-                 if ( V == 0X80 ) {
-                   for (i=0;i<5;i++) V = getc(fileP);
-                 }
-                 else ungetc(V,fileP);
-               }
-               filterFile.data.fileP = T1eexec(inputP->data.fileP);
-               if (filterFile.data.fileP == NULL) {
-                 fclose(inputFile.data.fileP);
-                 return(SCAN_FILE_OPEN_ERROR);
-               }
-               inputP = &filterFile;
- 
-               WantFontInfo = FALSE;
+	      /* We check for the pfb-headers in I/O-module */ 
+	      filterFile.data.fileP = T1eexec(inputP->data.fileP);
+	      if (filterFile.data.fileP == NULL) {
+		fclose(inputFile.data.fileP);
+		return(SCAN_FILE_OPEN_ERROR);
+	      }
+	      inputP = &filterFile;
+	      
+	      WantFontInfo = FALSE;
             }
         break;
     }
