@@ -1307,6 +1307,18 @@ void postprocess_project(int version)
             if (version < 50002) {
                 g[gno].p[setno].errbar.barsize *= 2;
             }
+            if (version < 50105) {
+                /* Starting with 5.1.5, X axis min & inverting is honored
+                   in pie charts */
+                if (get_graph_type(gno) == GRAPH_PIE) {
+                    world w;
+                    get_graph_world(gno, &w);
+                    w.xg1 = 0.0;
+                    w.xg2 = 2*M_PI;
+                    set_graph_world(gno, w);
+                    set_graph_xinvert(gno, FALSE);
+                }
+            }
         }
         for (naxis = 0; naxis < MAXAXES; naxis++) {
 	    tickmarks *t = get_graph_tickmarks(gno, naxis);
