@@ -571,6 +571,7 @@ int main(int argc, char *argv[])
 		        if ((resfp = grace_openw(argv[i])) == NULL) {
 		            exit(1);
 		        }
+			setvbuf(resfp, NULL, _IOLBF, 0);
 		    }
 		} else if (argmatch(argv[i], "-saveall", 8)) {
 		    i++;
@@ -869,6 +870,16 @@ static void VersionInfo(void)
 #else
     fprintf(stdout, "NetCDF support: off\n");
 #endif
+#ifdef HAVE_LIBPNG
+fprintf(stdout, "libpng: %s\n", bi_pnglib());
+#endif
+#ifdef HAVE_LIBJPEG
+fprintf(stdout, "libjpeg: %s\n", bi_libjpeg());
+#endif
+#ifdef HAVE_LIBPDF
+fprintf(stdout, "PDFlib: %s\n", bi_libpdf());
+#endif
+
 
 #ifdef DEBUG
     fprintf(stdout, "Debugging: enabled\n");
