@@ -206,12 +206,7 @@ static XtResource resources[] =
      (XtPointer) TRUE}
 };
 
-String fallbackResources[] = {
-    "XMgrace*mainWin.width: 680",
-    "XMgrace*mainWin.height: 700",
-    "XMgrace*fontList:-adobe-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
-    "XMgrace*tabFontList:-adobe-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
-    "XMgrace.consoleDialog*text.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*-*",
+String fallbackResourcesCommon[] = {
     "XMgrace.consoleDialog*text.columns: 72",
     "XMgrace.consoleDialog*text.rows: 5",
     "XMgrace*background: #e5e5e5",
@@ -262,6 +257,28 @@ String fallbackResources[] = {
     NULL
 };
 
+String fallbackResourcesHighRes[] = {
+    "XMgrace*mainWin.width: 680",
+    "XMgrace*mainWin.height: 700",
+    "XMgrace*fontList:-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
+    "XMgrace*tabFontList:-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*",
+    "XMgrace.consoleDialog*text.fontList:-*-courier-medium-r-normal-*-12-*-*-*-*-*-*-*",
+    "XMgrace*HContainer.marginHeight: 3",
+    "XMgrace*VContainer.marginHeight: 3",
+    NULL
+};
+
+String fallbackResourcesLowRes[] = {
+    "XMgrace*mainWin.width: 530",
+    "XMgrace*mainWin.height: 545",
+    "XMgrace*fontList:-*-helvetica-medium-r-normal-*-8-*-*-*-*-*-*-*",
+    "XMgrace*tabFontList:-*-helvetica-medium-r-normal-*-8-*-*-*-*-*-*-*",
+    "XMgrace.consoleDialog*text.fontList:-*-courier-medium-r-normal-*-8-*-*-*-*-*-*-*",
+    "XMgrace*HContainer.marginHeight: 1",
+    "XMgrace*VContainer.marginHeight: 1",
+    NULL
+};
+
 /*
  * main menubar
  */
@@ -319,8 +336,8 @@ static int is_motif_compatible(void)
     /* Finally, if LessTif is used, check for a reasonably new release */
     if (rt_lesstif) {
         s = strstr(_XmVersionString, "Version");
-        if (s == NULL || (strcmp(s, "Version 0.90.0") < 0)) {
-            errmsg("An old version of LessTif, please upgrade to 0.90.0 at least");
+        if (s == NULL || (strcmp(s, "Version 0.92.0") < 0)) {
+            errmsg("An old version of LessTif, please upgrade to 0.92.0 at least");
         }
     }
 #endif
@@ -354,10 +371,50 @@ static int is_motif_compatible(void)
 int initialize_gui(int *argc, char **argv)
 {
     ApplicationData rd;
+    String *allResources, *resolResources;
+    int lowres = FALSE;
+    unsigned int i, n_common, n_resol;
+    char *display_name = NULL;
 
     XtToolkitInitialize();
     app_con = XtCreateApplicationContext();
-    XtAppSetFallbackResources(app_con, fallbackResources);
+    
+    /* Check if we're running in the low-resolution X */
+    for (i = 1; i < *argc - 1; i++) {
+        /* See if display name was specified in the args */
+        char *pattern = "-display";
+        if (strstr(pattern, argv[i]) == pattern) {
+            display_name = argv[i + 1];
+        }
+    }
+    disp = XOpenDisplay(display_name);
+    if (disp) {
+        Screen *screen;
+        screen = DefaultScreenOfDisplay(disp);
+        if (HeightOfScreen(screen) < 740) {
+            lowres = TRUE;
+        }
+        XCloseDisplay(disp);
+    }
+    
+    n_common = sizeof(fallbackResourcesCommon)/sizeof(String) - 1;
+    if (lowres) {
+        n_resol  = sizeof(fallbackResourcesLowRes)/sizeof(String) - 1;
+        resolResources = fallbackResourcesLowRes;
+    } else {
+        n_resol  = sizeof(fallbackResourcesHighRes)/sizeof(String) - 1;
+        resolResources = fallbackResourcesHighRes;
+    }
+    allResources = xmalloc((n_common + n_resol + 1)*sizeof(String));
+    for (i = 0; i < n_common; i++) {
+        allResources[i] = fallbackResourcesCommon[i];
+    }
+    for (i = 0; i < n_resol; i++) {
+        allResources[n_common + i] = resolResources[i];
+    }
+    allResources[n_common + n_resol] = NULL;
+    XtAppSetFallbackResources(app_con, allResources);
+    
     disp = XtOpenDisplay(app_con, NULL, NULL, "XMgrace", NULL, 0, argc, argv);
     if (disp == NULL) {
 	errmsg("Can't open display");
@@ -828,6 +885,7 @@ static Widget CreateMainMenuBar(Widget parent)
     CreateMenuButton(sub2menupane, "Log scale", '\0', load_example, "tlog.agr");
     CreateMenuButton(sub2menupane, "Log2 scale", '\0', load_example, "log2.agr");
     CreateMenuButton(sub2menupane, "Logit scale", '\0', load_example, "logit.agr");
+    CreateMenuButton(sub2menupane, "Reciprocal scale", '\0', load_example, "reciprocal.agr");
     CreateMenuButton(sub2menupane, "Error bars", '\0', load_example, "terr.agr");
     CreateMenuButton(sub2menupane, "Date/time axis formats", '\0', load_example, "times.agr");
     CreateMenuButton(sub2menupane, "Australia map", '\0', load_example, "au.agr");
@@ -854,6 +912,7 @@ static Widget CreateMainMenuBar(Widget parent)
     CreateMenuButton(sub2menupane, "Box plot", '\0', load_example, "boxplot.agr");
     CreateMenuButton(sub2menupane, "Vector map", '\0', load_example, "vmap.agr");
     CreateMenuButton(sub2menupane, "XY Size", '\0', load_example, "xysize.agr");
+    CreateMenuButton(sub2menupane, "XY Color", '\0', load_example, "xycolor.agr");
 
     sub2menupane = CreateMenu(submenupane, "Type setting", 'T', FALSE);
     CreateMenuButton(sub2menupane, "Simple", '\0', load_example, "test2.agr");
